"""
Please refer to the documentation provided in the README.md,
which can be found on PyPI:

  https://pypi.org/project/abbreviate-names/
"""

VERSION = (0, 1, 2)
__version__ = ".".join([str(x) for x in VERSION])
